"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.executeLintAnalysis = executeLintAnalysis;
var _eslint = require("eslint");
var _path = _interopRequireWildcard(require("path"));
var _fileUtils = require("../../utils/fileUtils");
var _getFileUtils = require("./getFileUtils");
var _filterReports = require("./filterReports");
var _logger = require("../../utils/logger");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
/**
 * @function executeLintAnalysis - start lint analysis
 * @param {Object} cliParams -  Object represent cli Argument 
 * @returns {void}
 */

async function executeLintAnalysis(cliParams) {
  _logger.Logger.start('Start linting ...');
  var {
    env,
    dir = false
  } = cliParams;
  const {
    ruleConfigurationPath,
    impactBased,
    branchDiffPath
  } = (0, _fileUtils.getLintConfiguration)();
  if (env == 'ci' && dir || env == 'dev' && !dir) {
    let errorMessage = env == 'ci' ? 'expected valid  configuration' : env == 'dir' ? 'expected --dir path in cli' : 'issue in configuration';
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n running in ${env} mode. ${errorMessage}`);
    process.exit(0);
  }
  const reportPath = (0, _getFileUtils.getLintReportPath)();
  var {
    JsFiles,
    branch_diff_json
  } = (0, _getFileUtils.getFiles)(env, dir, branchDiffPath);
  const filteredLintReport = await startLint(branch_diff_json, JsFiles, env, impactBased, ruleConfigurationPath);
  _logger.Logger.stop();
  _logger.Logger.cliLog(_logger.Logger.SUCCESS_TYPE, `Linting analysis completed successfully...`);
  (0, _fileUtils.writeFileContents)(reportPath, JSON.stringify(filteredLintReport));
}

/**
 * @function startLint -  lint files based on configuration given
 * @param {JSON} branchDiff - Git Diff JSON  (This JSON get by git diff api)
 * @param {Array<String>} toLintFiles - impact files from git diff json
 * @param {string} linterFilesDirectory - directory path to lint
 * @param {string} impactBased  - param used to lint based on impact or not
 * @param {string} ruleConfigurationPath - configuration file path 
 * @returns {Promise<any|Object>}
 * 
 * @note -  while run via linterFilesDirectory impactBased param not used
 */

async function startLint(branchDiff = null, toLintFiles, env, impactBased, ruleConfigurationPath) {
  var eslint = createLint(ruleConfigurationPath);
  var rulesArrayReport = [];
  return await new Promise(async resolve => {
    if (env == 'dev' && toLintFiles.length) {
      rulesArrayReport = await eslint.lintFiles(toLintFiles);
    }
    if (env == 'ci' && branchDiff) {
      const filterJsonMap = (0, _filterReports.filterDiffJson)(branchDiff);
      for (let i = 0; i < toLintFiles.length; i++) {
        var singleFileReport = (await eslint.lintFiles(toLintFiles[i])).pop();
        if (impactBased === true) {
          const {
            filePath
          } = singleFileReport;
          singleFileReport = (0, _filterReports.filterLintReportBasedOnImpactLine)(singleFileReport, filterJsonMap, filePath);
        }
        rulesArrayReport.push(singleFileReport);
      }
    }
    resolve(rulesArrayReport);
  });
}

/**
 * @function createLint - create lint object 
 * @param {string} ruleConfigurationPath - configuration file path 
 * @returns {Object}
 */

function createLint(ruleConfigurationPath) {
  try {
    if (!(0, _fileUtils.checkIfFileExists)(ruleConfigurationPath)) {
      throw new Error('not Found');
    }
    return new _eslint.ESLint({
      overrideConfig: require((0, _getFileUtils.resolvePath)(ruleConfigurationPath)),
      cwd: _path.default.resolve(process.cwd()),
      useEslintrc: false
    });
  } catch (err) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Either Lint configuration not found or rule plugin are not installed ..`);
    process.exit(0);
  }
}