"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getBranchName = getBranchName;
exports.getCredentials = getCredentials;
exports.getGitUsername = getGitUsername;
exports.getProjectName = getProjectName;
exports.getProjectNameFrmGit = getProjectNameFrmGit;
exports.getRepoName = getRepoName;
exports.getSonarArtifactsDir = getSonarArtifactsDir;
exports.getSonarHostUrl = getSonarHostUrl;
exports.getSonarProperties = getSonarProperties;
exports.getSpawnOutput = getSpawnOutput;
var _rootPath = require("../../utils/rootPath");
var _path = _interopRequireDefault(require("path"));
var _child_process = require("child_process");
var _getFileUtils = require("../eslint/getFileUtils");
var _fileUtils = require("../../utils/fileUtils");
/**
 * @function getSonarHostUrl - get sonar host url  from property file
 * @returns {string}
 */

function getSonarHostUrl() {
  var {
    sonar_hostURL
  } = (0, _fileUtils.getLintConfiguration)();
  return sonar_hostURL;
}

/**
 * @function getProjectName - get  project name for sonarQube project
 * @param {Object} cliArgument -  Object represent cli Argument
 * @returns {string}
 */

function getProjectName(cliArgument) {
  var date = new Date();
  var timeSuffix = `${date.getUTCDate()}_${date.getHours()}_${`UTC`}`;
  const {
    dir = null,
    diffPath = null
  } = cliArgument;
  const default_project = getProjectNameFrmGit();
  var project_name = dir ? `${default_project}_dev` : diffPath ? `${default_project}_CI` : `${default_project}_dev_${timeSuffix}`;
  return project_name;
}
function getTestFiles(cliArgument) {
  return " ";
}
/**
 * @function getLintFiles - filter files based on cli argument
 * @param {Object} cliArgument -  Object represent cli Argument
 * @returns {Array<string>}
 */

function getLintFiles(cliArgument) {
  var {
    diffPath = false,
    dir = false
  } = cliArgument;
  const linterFilesDirectory = dir && (0, _fileUtils.checkIfFileExists)(dir) ? dir : false;
  if (linterFilesDirectory && !diffPath) {
    return (0, _fileUtils.getFilesPathFromRootDir)(linterFilesDirectory);
  }
  return (0, _getFileUtils.getFilesFromBranchDiff)(require((0, _getFileUtils.resolvePath)(diffPath)));
}

/**
 * @function getSonarProperties -  collect all sonar properties
 * @param {Object} cliObjects -  Object represent cli Argument
 * @returns {Array<Object>}
 */

function getSonarProperties(cliObjects) {
  const {
    env,
    dir
  } = cliObjects;
  const {
    branchDiffPath
  } = (0, _fileUtils.getLintConfiguration)();
  const {
    JsFiles,
    featureFiles
  } = (0, _getFileUtils.getFiles)(env, dir, branchDiffPath);
  const sonarqube_host_url = getSonarHostUrl();
  const sonar_scanner_path = (0, _rootPath.getSonarJarPath)();
  const {
    Dlogin,
    Dpassword
  } = getCredentials();
  var project_name = getProjectName(cliObjects);
  var resultFilesPath = featureFiles;
  var source_directories = JsFiles;
  const command = ["-jar", sonar_scanner_path, `-Dsonar.host.url=${sonarqube_host_url}`, `-Dsonar.projectKey=${project_name}`, `-Dsonar.login=${Dlogin}`, `-Dsonar.password=${Dpassword}`, `-Dsonar.sources=${source_directories}`, `-Dsonar.tests=${resultFilesPath}`, "-Dsonar.language=js", `-Dsonar.working.directory=${getSonarArtifactsDir()}`, `-Dsonar.eslint.reportPaths=${(0, _getFileUtils.getLintReportPath)()}`, "-Dsonar.javascript.lcov.reportPaths=coverage/lcov-report/lcov.info"];
  return command;
}

/**
 * @function getProjectNameFrmGit - get project from git
 * @returns {string}
 */

function getProjectNameFrmGit() {
  let $repoName = getRepoName();
  let $branchName = getBranchName();
  let $username = getGitUsername();
  return $repoName && $branchName && $username ? `${$repoName}_${$branchName}_${$username}` : "desk_default_projects";
}
/**
 * @function getGitUsername - get username from git
 * @returns {string}
 */

function getGitUsername() {
  const userName = getSpawnOutput("git", ["config", "user.email"]);
  return userName.pop().split("@")[0];
}

/**
 * @function getRepoName - get repo name from git
 * @returns {string}
 */

function getRepoName() {
  const repoName = getSpawnOutput("git", ["remote", "get-url", "origin"]);
  return repoName.pop().split("/").pop().split(".")[0];
}

/**
 * @function getBranchName - get branch name from git
 * @returns {string}
 */

function getBranchName() {
  // git rev-parse --abbrev-ref HEAD
  var branchName = getSpawnOutput("git", ["rev-parse", "--abbrev-ref", "HEAD"]); // to get current branch
  return branchName.shift().trim();
}

/**
 * @function getSpawnOutput -  run execution in spawn method
 * @param {string} command - prefix command
 * @param {Array<string>} params - command 
 * @returns {string | any}
 */

function getSpawnOutput(command, params = []) {
  const demon_process = (0, _child_process.spawnSync)(command, params, {
    shell: true
  });
  return demon_process.stdout.toString().split("\n").filter(Boolean);
}

/**
 * @function getCredentials -  get credentials for sonarQube
 * @returns {Object}
 */

function getCredentials() {
  const {
    loginName,
    password
  } = (0, _fileUtils.getLintConfiguration)();
  return {
    Dlogin: loginName,
    Dpassword: password
  };
}

/**
 * @function getSonarArtifactsDir - get artifacts path for sonarQube
 * @returns {string}
 */

function getSonarArtifactsDir() {
  return _path.default.resolve(process.cwd(), "lint-report", ".scannerwork");
}